% ######################
% global_vars a renseigner
% ######################

% start and end (0 means the last file) of the recording sequence
first_file=1
last_file=2

% name of the movie file
name_movie = 'contrarotating_wz_F0.6R3.mov';
framerate = 2;
quality = 1.;

% field to plot (vx=1,vy=2,vz=3,rho=4,wx=5,wy=6,wz=7)
type_field=1;


% strides along each direction (useful to use less memory when handling large simulations)
str_x=1;
str_y=1;
str_z=1;

% ######################
% on touche plus
% ######################

% noms des fichiers de donnees
if strcmp('PCWIN', computer)
	[poub,liste_fichiers] = system('dir /B');
else
	[poub,liste_fichiers] = system('ls ');
end

[start,finish] = regexpi(liste_fichiers,'velo_rho_vort.t=\d*\.\d*');

% initial_velocity_big_endian_mode_zigzag=0.01_512x512x256

nb_fichiers = size(start,2);

% we adjust the first and last file to process
if last_file == 0
    last_file=nb_fichiers;
end

clear nom_fichier
for i=1:nb_fichiers
    nom_fichier(i,:) = liste_fichiers(start(i):finish(i));
end

nom_fichier = sortrows(nom_fichier);
fichier = nom_fichier(1,:)

% we determine the type of binary data (little-endian, big-endian) with the file header

% we start with little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(fichier, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% if the flag is not 1, we try big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(fichier, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % we check again the value of the flag
    if flag ~= 1
        fprintf(1, '\n There is a problem with the format of the data file: the flag is not 1\n');
        return
    end
end


% we read the metadata
FID = fopen(fichier, 'r',type_endian);

poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb         = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

status = fclose(FID);

% we display some informations about the simulation
fprintf(1, '\n Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, '\n Physical dimensions of the box: %f * %f * %f \n', lx, ly, lz);

if (stratification == 0)
    freq_brunt = 0;
else
    freq_brunt = xns;
end

fprintf(1, '\n Stratification: %f \n', freq_brunt);

if (perturb==0)
    texte = ['non perturbative'];
else
    if (lin==0) then
        texte = ['perturbative/non linear\n'];
    else
        texte = ['perturbative/linear\n'];
    end
end

% we determine the number of points along each direction (with the stride)
nxp = 1+floor( (nx-1)/str_x);
nyp = 1+floor( (ny-1)/str_y);
nzp = 1+floor( (nz-1)/str_z);

fprintf(1, '\n Dimensions used for the plot: %i * %i * %i \n', nxp, nyp, nzp);


% on cre la grille des points de colocation
X = linspace(0, lx-lx/nxp, nxp);
Y = linspace(0, ly-ly/nyp, nyp);
Z = linspace(0, lz-lz/nzp, nzp);

scrsz = get(0,'ScreenSize');
figure('Position',[100 100 scrsz(3)*0.8 scrsz(4)*0.8])

MakeQTMovie('start',name_movie)
MakeQTMovie('framerate',framerate)
MakeQTMovie('quality',quality)

fprintf('\n Plotting...\n');

% we create the working arrays
Vort_Z = zeros(nxp,nyp,nzp);

for it=first_file:last_file

    fprintf(1, ' Num of frame: %i/%i \n', it,nb_fichiers);

    clf
    
    fprintf(1, '      Reading...');
    FID = fopen(nom_fichier(it,:),'r',type_endian);

    % on lit toutes les metadonnees d'un coup
    fseek(FID, 4*34, 'bof');
    instant = fread(FID, 1, 'real*8');

    % on va directement lire la vorticite verticale en tenant compte des strides

    % no stride, on lit de faon classique
    if (str_x==1 & str_y==1 & str_z==1)
        fseek(FID, 33*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4, 'bof');
        Vort_Z = fread(FID, nxp*nyp*nzp, 'real*8');
        Vort_Z = reshape(Vort_Z, [nxp,nyp,nzp]);
    else
        for izp=1:nzp
            for iyp=1:nyp
                fseek(FID,33*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4 + 8*(izp-1)*str_z*nx*ny + 8*(iyp-1)*str_y*nx, 'bof');
                Vort_Z(:,iyp,izp) = fread(FID, nxp, 'real*8', 8*(str_x-1));
            end
        end
    end

    status = fclose(FID);

    fprintf(1, '      Plotting...\n');
    pmoy_v=mean(max(max(Vort_Z)));
    
    % on stocke l'iso-valeur
    % pmoy_v=tab_pmoy(it);
    
    nmoy_v=-pmoy_v;

    p = patch(isosurface(Y,X,Z,Vort_Z,0.6*pmoy_v));
    isonormals(Y,X,Z,Vort_Z,p)
    set(p,'FaceAlpha',0.9,'FaceColor','red','EdgeColor','none');
    p2 = patch(isosurface(Y,X,Z,Vort_Z,0.1*pmoy_v));
    isonormals(Y,X,Z,Vort_Z,p2)
    set(p2,'FaceAlpha',0.15,'FaceColor',[255,251,140]/256,'EdgeColor','none');

    p3 = patch(isosurface(Y,X,Z,Vort_Z,0.6*nmoy_v));
    isonormals(Y,X,Z,Vort_Z,p3)
    set(p3,'FaceAlpha',0.9,'FaceColor','blue','EdgeColor','none');
    p4 = patch(isosurface(Y,X,Z,Vort_Z,0.1*nmoy_v));
    isonormals(Y,X,Z,Vort_Z,p4)
    set(p4,'FaceAlpha',0.15,'FaceColor',[130,186,212]/256,'EdgeColor','none');

    daspect([1 1 1])
    view(3);
    axis([Y(1) Y(end) X(1) X(end) Z(1) Z(end)])
    box on
    grid on
    camlight
    lighting gouraud
    
    title(['Run F0.6R3     t=', 8*num2str(instant)], 'FontSize',30);
	set(gca,'Linewidth',1.5)
    set(gca,'XTick',10000)
    set(gca,'YTick',10000)
    set(gca,'ZTick',10000)
       
    MakeQTMovie('addfigure');

end

MakeQTMovie('finish')
MakeQTMovie('cleanup');
