% ######################
% global_vars a renseigner
% ######################

% field to plot (vx=1,vy=2,vz=3,rho=4,wx=5,wy=6,wz=7)
type_field=7;

% strides along each direction (useful to use less memory when handling large simulations)
str_x=4;
str_y=4;
str_z=2;

% Do we save the figure .fig in a directory 3D_views ?
save_file=1;

% Est-ce qu'on trace un plan de coupe ? Si oui, on donne la position en x de ce plan de coupe
plan_coupe=0;
x_coupe = 4.2-1.05-0.525;

% interpolation en z (c'est une interpolation FFT)
facteur_interp_z = 0;

% Est-ce que l'on clippe dans le plan xy le champ total pour ne conserver qu'une seul zone intéressante ?
clipping=0;
x_cl=0;
x_cu=4.2;
y_cl=1.;
y_cu=4.2;



% ######################
% on touche plus
% ######################

if strcmp('PCWIN', computer)
	system('dir /O-D /AD /B .');
else
	system('ls -t .');
end

fichier = input('\nFichier de vitesse a representer: ', 's');


% we determine the type of binary data (little-endian, big-endian) with the file header

% we start with little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(fichier, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% if the flag is not 1, we try big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(fichier, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % we check again the value of the flag
    if flag ~= 1
        fprintf(1, '\n There is a problem with the format of the data file: the flag is not 1\n');
        return
    end
end


% we read the metadata
FID = fopen(fichier, 'r',type_endian);

poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb         = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

status = fclose(FID);

% we display some informations about the simulation
fprintf(1, '\n Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, '\n Physical dimensions of the box: %f * %f * %f \n', lx, ly, lz);

if (stratification == 0)
    freq_brunt = 0;
else
    freq_brunt = xns;
end

fprintf(1, '\n Stratification: %f \n', freq_brunt);

if (perturb==0)
    texte = ['non perturbative'];
else
    if (lin==0) then
        texte = ['perturbative/non linear\n'];
    else
        texte = ['perturbative/linear\n'];
    end
end

% we determine the number of points along each direction (with the stride)
nxp = 1+floor( (nx-1)/str_x);
nyp = 1+floor( (ny-1)/str_y);
nzp = 1+floor( (nz-1)/str_z);

fprintf(1, '\n Dimensions used for the plot: %i * %i * %i \n', nxp, nyp, nzp);


% on crée la grille des points de colocation
X = linspace(0, lx-lx/nxp, nxp);
Y = linspace(0, ly-ly/nyp, nyp);
Z = linspace(0, lz-lz/nzp, nzp);

scrsz = get(0,'ScreenSize');
figure('Position',[100 100 scrsz(3)*0.8 scrsz(4)*0.8])

fprintf('\n Plotting...\n');

fprintf(1, '      Reading...');
FID = fopen(fichier,'r',type_endian);

% on lit toutes les metadonnees d'un coup
fseek(FID, 4*34, 'bof');
instant = fread(FID, 1, 'real*8');

% on va directement lire la vorticite verticale en tenant compte des strides

% no stride, on lit de façon classique
if (str_x==1 & str_y==1 & str_z==1)
	fseek(FID, 33*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4, 'bof');
	field = fread(FID, nxp*nyp*nzp, 'real*8');
	field = reshape(field, [nxp,nyp,nzp]);
else
	field = zeros(nxp,nyp,nzp);
	for izp=1:nzp
		for iyp=1:nyp
            fseek(FID,33*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4 + 8*(izp-1)*str_z*nx*ny + 8*(iyp-1)*str_y*nx, 'bof');
            field(:,iyp,izp) = fread(FID, nxp, 'real*8', 8*(str_x-1));
		end
    end
end

status = fclose(FID);

% on trace un plan de coupe
if (plan_coupe==1)
    hold on
    nxl = floor(nxp/4);
    nxu = floor(3*nxp/4);
    nyl = floor(nyp/4);
    nyu = floor(3*nyp/4);
    
    [xx,yy,zz] = meshgrid(X(nxl:nxu),Y(nyl:nyu),Z);
    xslice = [x_coupe]; yslice = []; zslice = [];
    slice(xx,yy,zz,field(nxl:nxu,nyl:nyu,:),xslice,yslice,zslice)
    shading interp
end


% we cut the field to the area of interest
if (clipping==1)
    nx_cl=max(floor(x_cl/4.2*nxp),1);
    nx_cu=min(floor(x_cu/4.2*nxp),nxp);

    ny_cl=max(floor(x_cl/4.2*nyp),1);
    ny_cu=min(floor(x_cu/4.2*nyp),nyp);

    field = field(nx_cl:nx_cu,ny_cl:ny_cu,:);
else
    nx_cl=1;
    nx_cu=nxp;

    ny_cl=1;;
    ny_cu=nyp;
end


% we interpolate the field if required
if (facteur_interp_z > 1)
	fprintf(1, '\n      Interpolating fields...\n');
    
    field = reshape(field, [(nx_cu-nx_cl+1)*(ny_cu-ny_cl+1),nzp]);
    field = interpft(field, facteur_interp_z*nzp,2);
    field = reshape(field, [(nx_cu-nx_cl+1), (ny_cu-ny_cl+1), nzp*facteur_interp_z]);
    
    % on crée la grille des points de colocation
    X = linspace(0, lx-lx/nxp, nxp);
    Y = linspace(0, ly-ly/nyp, nyp);
    Z = linspace(0, lz-lz/(nzp*facteur_interp_z), nzp*facteur_interp_z);
end


% on shifte pour centrer la figure
% field = circshift(field,[-40,0,0]);

fprintf(1, '      Plotting...\n');

pmoy_v=mean(max(max(field)))

%fprintf(1, '      On impose artificiellement le niveau des isosurface...\n');
%pmoy_v=1.41

nmoy_v=-pmoy_v;

p = patch(isosurface(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,0.6*pmoy_v));
isonormals(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,p)
set(p,'FaceAlpha',0.9,'FaceColor','red','EdgeColor','none');
p2 = patch(isosurface(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,0.1*pmoy_v));
isonormals(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,p2)
set(p2,'FaceAlpha',0.15,'FaceColor',[255,251,140]/256,'EdgeColor','none');

p3 = patch(isosurface(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,0.6*nmoy_v));
isonormals(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,p3)
set(p3,'FaceAlpha',0.9,'FaceColor','blue','EdgeColor','none');
p4 = patch(isosurface(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,0.1*nmoy_v));
isonormals(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,p4)
set(p4,'FaceAlpha',0.15,'FaceColor',[130,186,212]/256,'EdgeColor','none');

daspect([1 1 1])
view(3);
axis([Y(1) Y(end) X(1) X(end) Z(1) Z(end)])
camlight
lighting gouraud
axis on
box off
set(gca,'Visible','on')


% on sauvegarde le resultat dans un fichier .fig
if (save_file==1)
    !mkdir -p 3D_views
    saveas(gcf,['3D_views/3D_view.t=',num2str(instant),'.fig'],'fig')
end
