program main

    use timing

    implicit none

    ! dimensions of the data file
    integer, parameter :: nx=128
    integer, parameter :: ny=128
    integer, parameter :: nz=64
    double precision, parameter :: vx_translation=0.
    double precision, parameter :: vy_translation=0.

    ! maximum number of data files
    integer, parameter :: nb_file_max=200
    character(len=200) :: nom_fichier
    integer :: nb_file
    character(len=200), dimension(nb_file_max) :: file_name
    double precision, dimension(nb_file_max) :: instants

    ! slices of data
    double precision, dimension(0:nx-1,0:ny-1) :: vx_iz, vy_iz, vz_iz, rho_iz, wx_iz, wy_iz, wz_iz

    ! kinetic (horizontal, vertical, total), potential and enstrophy energy (horizontal, vertical, total)
    double precision, dimension(nb_file_max, 0:nz-1) :: energy_h, energy_v, energy_t, pot_energy, enstrophy_h, enstrophy_v, enstrophy_t


    ! physical dimensions of the box
    double precision :: lx,ly,lz

    integer :: eof
    integer :: i_file, ix,iy,iz

    write(*,*) '************************************'
    write(*,*) '******      ENERGY SLICE      ******'
    write(*,*) '************************************'

    ! On lit les noms des fichiers a traiter
    write(*,*) 'Fichier contenant les noms des fichiers a analyser :'
    read(*,*) nom_fichier
    write(*,*)

    open (unit=8,file=nom_fichier,form="formatted",status="old",action="read",position="rewind",iostat=eof)
    nb_file = 0
    do while (eof==0)
        nb_file = nb_file + 1
        read(8,*,iostat=eof)  file_name(nb_file)
    end do
    close(8)

    nb_file = nb_file - 1

    do i_file=1, nb_file
        write(*,*) '    ', trim(file_name(i_file))
    end do

    write(*,*)
    write(*,*) '   --> Number of data files: ', nb_file


    ! on lit les meta donnees dans le premier fichier (dimensions de la boite...)
    call read_metadata(file_name(1))

    write(*,*)
    write(*,*) 'Number of points: ', nx, ny, nz
    write(*,*) 'Dimensions: ', lx, ly, lz
    write(*,*)

    do i_file=1,nb_file

        ! we determine the time of the record
        open (unit=8,file=file_name(i_file),access='direct',form='unformatted',recl=34)
        read(8,rec=2) instants(i_file)
        close(8)
        write(*,*)
        write(*,*) 'time=',instants(i_file)

        do iz=0,nz-1

            ! Kinetic energy and enstrophy
            call timer_start('energy')

            call read_slice_data(file_name(i_file), iz, vx_iz,'vx')
            call read_slice_data(file_name(i_file), iz, vy_iz,'vy')
            call read_slice_data(file_name(i_file), iz, vz_iz,'vz')
            call read_slice_data(file_name(i_file), iz, rho_iz,'rh')
            call read_slice_data(file_name(i_file), iz, wx_iz,'wx')
            call read_slice_data(file_name(i_file), iz, wy_iz,'wy')
            call read_slice_data(file_name(i_file), iz, wz_iz,'wz')

            call get_energy_enstrophy(vx_iz, vy_iz, vz_iz, rho_iz, wx_iz, wy_iz, wz_iz, &
                                      energy_h(i_file,iz), energy_v(i_file,iz), energy_t(i_file,iz), &
                                      pot_energy(i_file,iz), &
                                      enstrophy_h(i_file,iz), enstrophy_v(i_file,iz), enstrophy_t(i_file,iz))

            call timer_stop('energy')

        end do
    end do

    open (unit=76,file='energy_results_'//nom_fichier,form='unformatted')
    write(76) 1,nx,ny,nz,lx,ly,lz
    write(76) nb_file
    write(76) instants(1:nb_file)
    write(76) energy_h(1:nb_file,:)
    write(76) energy_v(1:nb_file,:)
    write(76) energy_t(1:nb_file,:)
    write(76) pot_energy(1:nb_file,:)
    write(76) enstrophy_h(1:nb_file,:)
    write(76) enstrophy_v(1:nb_file,:)
    write(76) enstrophy_t(1:nb_file,:)
    close(76)

    call timer_report_all

    contains


!**************************
    subroutine get_energy_enstrophy(vx_iz, vy_iz, vz_iz, rho_iz, wx_iz, wy_iz, wz_iz, energy_h_iz, energy_v_iz, &
                                    energy_t_iz, pot_energy_iz, enstrophy_h_iz, enstrophy_v_iz, enstrophy_t_iz)
!**************************

        implicit none

        double precision, dimension(0:nx-1,0:ny-1), intent(in) :: vx_iz, vy_iz, vz_iz, rho_iz, wx_iz, wy_iz, wz_iz
        double precision, intent(out) :: energy_h_iz, energy_v_iz, energy_t_iz, pot_energy_iz
        double precision, intent(out) :: enstrophy_h_iz, enstrophy_v_iz, enstrophy_t_iz

        integer :: ix,iy

        energy_h_iz = 0
        energy_v_iz = 0
        energy_t_iz = 0
        pot_energy_iz = 0
        enstrophy_h_iz = 0
        enstrophy_v_iz = 0
        enstrophy_t_iz = 0

        do iy=0, ny-1
          do ix=0, nx-1
            energy_h_iz = energy_h_iz + (vx_iz(ix,iy)+vx_translation)**2 + (vy_iz(ix,iy)+vy_translation)**2
            energy_v_iz = energy_v_iz + vz_iz(ix,iy)**2

            pot_energy_iz = pot_energy_iz + rho_iz(ix,iy)**2

            enstrophy_h_iz = enstrophy_h_iz + wx_iz(ix,iy)**2 + wy_iz(ix,iy)**2
            enstrophy_v_iz = enstrophy_v_iz + wz_iz(ix,iy)**2
          end do
        end do

        energy_h_iz = lx*ly/(2*nx*ny)*energy_h_iz
        energy_v_iz = lx*ly/(2*nx*ny)*energy_v_iz
        pot_energy_iz = lx*ly/(2*nx*ny)*pot_energy_iz
        enstrophy_h_iz = lx*ly/(2*nx*ny)*enstrophy_h_iz
        enstrophy_v_iz = lx*ly/(2*nx*ny)*enstrophy_v_iz

        energy_t_iz = energy_h_iz + energy_v_iz
        enstrophy_t_iz = enstrophy_h_iz + enstrophy_v_iz

end subroutine


!**************************
    subroutine read_metadata(file_name)
!**************************
!    we read the metadata and we check the dimensions in the file

        implicit none

        character(len=200), intent(in) :: file_name
        integer :: flag, nxread, nyread, nzread

        ! we check the dimensions
        open (unit=8,file=file_name,form='unformatted')
        read(8) flag,nxread,nyread,nzread,lx,ly,lz
        close(8)
        if (flag /= 1 .or. nxread /= nx .or. nyread /= ny .or. nzread /= nz) then
            write(*,*) "ERROR: the data file has bad format. Failure."
            stop
        end if

    end subroutine



!**************************
    subroutine read_slice_data(file_name,iz, data_field,name_field)
!**************************

        implicit none

        character(len=200), intent(in) :: file_name
        integer, intent(in) :: iz
        double precision, dimension(0:nx-1,0:ny-1),intent(out) :: data_field
        character(len=2), intent(in) :: name_field
        integer :: position_record
        integer, dimension(31) :: trash_int
        double precision :: trash_double

        ! cas particulier a cause des fonctions io de Fortran pas top top...
        if (name_field == 'vx' .and. iz==0) then
            ! we open the data file again and we go directly to the data slice of interest
            open (unit=8,file=file_name,form='unformatted')
            read(8) trash_int
            read(8) trash_double
            read(8) data_field
            close(8)
        else

            if (name_field == 'vx')  then
                position_record = 38+0*2*(1+nx*ny*nz)+2*iz*nx*ny
            else if (name_field == 'vy') then
                position_record = 38+1*2*(1+nx*ny*nz)+2*iz*nx*ny
            else if (name_field == 'vz') then
                position_record = 38+2*2*(1+nx*ny*nz)+2*iz*nx*ny
            else if (name_field == 'rh') then
                position_record = 38+3*2*(1+nx*ny*nz)+2*iz*nx*ny
            else if (name_field == 'wx') then
                position_record = 38+4*2*(1+nx*ny*nz)+2*iz*nx*ny
            else if (name_field == 'wy') then
                position_record = 38+5*2*(1+nx*ny*nz)+2*iz*nx*ny
            else if (name_field == 'wz') then
                position_record = 38+6*2*(1+nx*ny*nz)+2*iz*nx*ny
            end if

            ! we open the data file again and we go directly to the data slice of interest
            open (unit=8,file=file_name,access='direct',form='unformatted',recl=position_record)
            read(8,rec=2) data_field
            close(8)
        end if

    end subroutine

end program
