% ######################
% global_vars a renseigner
% ######################

% base directory
dir_base ='.';

% start and end (0 means the last file) of the recording sequence
first_file=1
last_file=0

% name of the movie file
name_movie = 'contrarotating.mov';
framerate = 2;
quality = 0.85;

% ######################
% on touche plus
% ######################

fprintf(1, '\nBase directory: %s\n\n', dir_base);

if strcmp('PCWIN', computer)
	system(['dir /O-D /AD /B ', dir_base]);
else
	system(['ls -t ', dir_base]);
end

% dir_spec = input('\nSpecific directory: ', 's');

% dir_complet = [dir_base,'/',dir_spec,'/'];
dir_complet = dir_base;


% noms des fichiers de donnees
if strcmp('PCWIN', computer)
	[poub,liste_fichiers] = system(['dir /B "', dir_complet, '"']);
else
	[poub,liste_fichiers] = system(['ls ', dir_complet]);
end

[start,finish] = regexpi(liste_fichiers,'velo_rho_vort.t=\d*\.\d*');

nb_fichiers = size(start,2);

% we adjust the first and last file to process
if last_file == 0
    last_file=nb_fichiers;
end

clear nom_fichier
for i=1:nb_fichiers
    nom_fichier(i,:) = liste_fichiers(start(i):finish(i));
end

nom_fichier = sortrows(nom_fichier);
fichier = nom_fichier(1,:)

% we determine the type of binary data (little-endian, big-endian) with the file header

% we start with little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(fichier, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% if the flag is not 1, we try big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(fichier, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % we check again the value of the flag
    if flag ~= 1
        fprintf(1, '\n There is a problem with the format of the data file: the flag is not 1\n');
        return
    end
end


% we read the metadata
FID = fopen(fichier, 'r',type_endian);

poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb         = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

status = fclose(FID);

% we display some informations about the simulation
fprintf(1, '\n Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, '\n Physical dimensions of the box: %f * %f * %f \n', lx, ly, lz);

if (stratification == 0)
    freq_brunt = 0;
else
    freq_brunt = xns;
end

fprintf(1, '\n Stratification: %f \n', freq_brunt);

if (perturb==0)
    texte = ['non perturbative'];
else
    if (lin==0) then
        texte = ['perturbative/non linear\n'];
    else
        texte = ['perturbative/linear\n'];
    end
end

% on cre la grille des points de colocation
X = linspace(0, lx-lx/nx, nx);
Y = linspace(0, ly-ly/ny, ny);
Z = linspace(0, lz-lz/nz, nz);

% we create the wavevectors
clear kx ky kz
for ik=0:floor(nx/2)
    kx(ik+1) = ik*2*pi/lx;
end
for ik=floor(nx/2)+1:nx-1
    kx(ik+1) = (ik-nx)*2*pi/lx;
end

for ik=0:floor(ny/2)
    ky(ik+1) = ik*2*pi/ly;
end
for ik=floor(ny/2)+1:ny-1
    ky(ik+1) = (ik-ny)*2*pi/ly;
end

for ik=0:floor(nz/2)
    kz(ik+1) = ik*2*pi/lz;
end
for ik=floor(nz/2)+1:nz-1
    kz(ik+1) = (ik-nz)*2*pi/lz;
end

scrsz = get(0,'ScreenSize');
figure('Position',[100 100 scrsz(3)*0.8 scrsz(4)*0.8])

MakeQTMovie('start',name_movie)
MakeQTMovie('framerate',framerate)
MakeQTMovie('quality',quality)

clear senergy_kz energymax kzmax gamma instant

fprintf('\n First pass: calculus of energymax, kzmax...\n');

% on cre les 2 fichiers de travail qui vont tre utiliss
work1 = zeros(nx,ny,nz);
work2 = zeros(nx,ny,nz);
senergy = zeros(nx,ny,nz);

for it=first_file:last_file

    local_it = it-first_file+1;

    fprintf(1, ' Num of frame: %i/%i \n', it,nb_fichiers);

    clf

    % on lit les donnees dans le dernier fichier enregistre
    FID = fopen([dir_complet,'/',nom_fichier(it,:)], 'r',type_endian);

    % on lit toutes les metadonnees d'un coup "comme si" c'tait des entiers de 4 octets mme si ce n'est pas vrai,
    % l'important est que le nombre d'octets soit respecte
    poub = fread(FID, 33, 'integer*4');

    poub = fread(FID, 1, 'integer*4');
    instant(local_it) = fread(FID, 1, 'real*8');

    % vitesse suivant x
    poub = fread(FID, 2, 'integer*4');
    work1 = fread(FID, nx*ny*nz, 'real*8');
    work1 = reshape(work1,nx,ny,nz);
    work2 = fftn(work1);
    senergy = 0.5*(real(work2).^2 + imag(work2).^2);

    % we determine gamma = Fh.Re^0.5 to estimate the relative importance of viscosity and buoyancy forces
    gamma(local_it) = mean(max(max(abs(work1))));

    % vitesse suivant y
    poub = fread(FID, 2, 'integer*4');
    work1 = fread(FID, nx*ny*nz, 'real*8');
    work1 = reshape(work1,nx,ny,nz);
    work2 = fftn(work1);
    senergy = senergy + 0.5*(real(work2).^2 + imag(work2).^2);

    % we determine gamma = Fh.Re^0.5 to estimate the relative importance of viscosity and buoyancy forces
    gamma(local_it) = max( gamma(local_it) , mean(max(max(abs(work1)))))^(3/2)/(freq_brunt*nu^0.5);

    % vitesse suivant z
    poub = fread(FID, 2, 'integer*4');
    work1 = fread(FID, nx*ny*nz, 'real*8');
    work1 = reshape(work1,nx,ny,nz);
    work2 = fftn(work1);
    senergy = senergy + 0.5*(real(work2).^2 + imag(work2).^2);

    poub = fread(FID, 2, 'integer*4');
    work1 = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    work1 = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    work1 = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    work1 = fread(FID, nx*ny*nz, 'real*8');

    status = fclose(FID);

    size(senergy)

    senergy_kz(:,local_it) = squeeze(sum(sum(senergy,1),2));
    clear senergy;

    [energymax(local_it),ind_kzmax] = max(senergy_kz(2:nz/2+1,local_it));
    kzmax(local_it) = kz(ind_kzmax+1);

end

fprintf('\n Second pass: plotting...\n');

% we create the working arrays
clear senergy work2 ;
Vort_Z = zeros(nx,ny,nz);

for it=first_file:last_file

    local_it = it-first_file+1;

    fprintf(1, ' Num of frame: %i/%i \n', it,nb_fichiers);

    clf

    % on lit les donnees dans le dernier fichier enregistre
    FID = fopen([dir_complet,'/',nom_fichier(it,:)], 'r',type_endian);

    % on lit toutes les metadonnees d'un coup "comme si" c'tait des entiers de 4 octets mme si ce n'est pas vrai,
    % l'important est que le nombre d'octets soit respecte
    poub = fread(FID, 33, 'integer*4');

    poub = fread(FID, 1, 'integer*4');
    poub = fread(FID, 1, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    work1 = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    work1 = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    work1 = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    work1 = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    work1 = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    work1 = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Vort_Z = fread(FID, nx*ny*nz, 'real*8');

    status = fclose(FID);

    Vort_Z = reshape(Vort_Z, [nx,ny,nz]);

    pmoy_v=mean(max(max(Vort_Z)));
    nmoy_v=-pmoy_v;

    subplot(3,3,[1 2 4 5])

    p = patch(isosurface(Y,X,Z,Vort_Z,0.6*pmoy_v));
    isonormals(Y,X,Z,Vort_Z,p)
    set(p,'FaceAlpha',0.9,'FaceColor','red','EdgeColor','none');
    p2 = patch(isosurface(Y,X,Z,Vort_Z,0.05*pmoy_v));
    isonormals(Y,X,Z,Vort_Z,p2)
    set(p2,'FaceAlpha',0.15,'FaceColor',[255,251,140]/256,'EdgeColor','none');

    p3 = patch(isosurface(Y,X,Z,Vort_Z,0.6*nmoy_v));
    isonormals(Y,X,Z,Vort_Z,p3)
    set(p3,'FaceAlpha',0.9,'FaceColor','blue','EdgeColor','none');
    p4 = patch(isosurface(Y,X,Z,Vort_Z,0.05*nmoy_v));
    isonormals(Y,X,Z,Vort_Z,p4)
    set(p4,'FaceAlpha',0.15,'FaceColor',[130,186,212]/256,'EdgeColor','none');

    daspect([1 1 1])
    view(3);
    axis([Y(1) Y(end) X(1) X(end) Z(1) Z(end)])
    box on
    grid on
    camlight
    lighting gouraud
    % lighting phong

    title({['Vertical vorticity    t=', num2str(instant(local_it)), '    Frame=',num2str(it),'/',num2str(nb_fichiers)]; ['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  dt=',num2str(dt), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte]},'Interpreter','none');

    subplot(3,3,[7]);
    plot(kz(2:nz/2),senergy_kz(2:nz/2,local_it),'linewidth',1.);
    xlabel('kz');
    title(['Energy t=', num2str(instant(local_it))],'Interpreter','none');

    subplot(3,3,[3]);
    plot(instant,energymax,'-',instant(local_it),energymax(local_it),'r*');
    xlabel('time');
    title('Energy_max','Interpreter','none');

    subplot(3,3,[6]);
    plot(instant,kzmax,'-',instant(local_it),kzmax(local_it),'r*');
    xlabel('time');
    temp = ylim;
    ylim([0 temp(2)]);
    title('kz(energy_max)','Interpreter','none');

    subplot(3,3,[9]);
    plot(instant,gamma,'-',instant(local_it),gamma(local_it),'r*');
    xlabel('time');
    title('Fr.Re^{1/2} with L=cte=1.');


    MakeQTMovie('addfigure');

end

MakeQTMovie('finish')
MakeQTMovie('cleanup');

clear all;
