% ######################
% global_vars a renseigner
% ######################

% base directory
dir_base ='.';


% ######################
% on touche plus
% ######################

fprintf(1, '\nBase directory: %s\n\n', dir_base);

if strcmp('PCWIN', computer)
	system(['dir /O-D /AD /B ', dir_base]);
else
	system(['ls -t ', dir_base]);
end

dir_spec = input('\nSpecific directory: ', 's');

dir_complet = [dir_base,'/',dir_spec,'/'];


% noms des fichiers de donnees
if strcmp('PCWIN', computer)
	[poub,liste_fichiers] = system(['dir /B "', dir_complet, '"']);
else
	[poub,liste_fichiers] = system(['ls ', dir_complet]);
end

[start,finish] = regexpi(liste_fichiers,'velo_rho_vort.t=\d*\.\d*');

nb_fichiers = size(start,2);

clear nom_fichier
for i=1:nb_fichiers
    nom_fichier(i,:) = liste_fichiers(start(i):finish(i));
end

nom_fichier = sortrows(nom_fichier);
fichier = nom_fichier(1,:)

% we determine the type of binary data (little-endian, big-endian) with the file header

% we start with little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(fichier, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% if the flag is not 1, we try big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(fichier, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % we check again the value of the flag
    if flag ~= 1
        fprintf(1, '\n There is a problem with the format of the data file: the flag is not 1\n');
        return
    end
end


% we read the metadata
FID = fopen(fichier, 'r',type_endian);

poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb        = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

status = fclose(FID);

% we display some informations about the simulation
fprintf(1, '\n Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, '\n Physical dimensions of the box: %f * %f * %f \n', lx, ly, lz);

if (stratification == 0)
    freq_brunt = 0;
else
    freq_brunt = xns;
end

fprintf(1, '\n Stratification: %f \n', freq_brunt);

if (perturb==0)
    texte = ['non perturbative'];
else
    if (lin==0) then
        texte = ['perturbative/non linear\n'];
    else
        texte = ['perturbative/linear\n'];
    end
end

% on cre la grille des points de colocation
X = linspace(0, lx-lx/nx, nx);
Y = linspace(0, ly-ly/ny, ny);
Z = linspace(0, lz-lz/nz, nz);

% we create the wavevectors
for ik=0:floor(nx/2)
    kx(ik+1) = ik*2*pi/lx;
end
for ik=floor(nx/2)+1:nx-1
    kx(ik+1) = (ik-nx)*2*pi/lx;
end

for ik=0:floor(ny/2)
    ky(ik+1) = ik*2*pi/ly;
end
for ik=floor(ny/2)+1:ny-1
    ky(ik+1) = (ik-ny)*2*pi/ly;
end

for ik=0:floor(nz/2)
    kz(ik+1) = ik*2*pi/lz;
end
for ik=floor(nz/2)+1:nz-1
    kz(ik+1) = (ik-nz)*2*pi/lz;
end


scrsz = get(0,'ScreenSize');
figure('Position',[1 scrsz(4) scrsz(3) scrsz(4)])

for it=1:nb_fichiers

    fprintf(1, 'Num of frame: %i/%i \n', it,nb_fichiers);

    clf

    % on lit les donnees dans le dernier fichier enregistre
    FID = fopen([dir_complet,'/',nom_fichier(it,:)], 'r',type_endian);

    % on lit toutes les metadonnees d'un coup "comme si" c'tait des entiers de 4 octets mme si ce n'est pas vrai,
    % l'important est que le nombre d'octets soit respecte
    poub = fread(FID, 33, 'integer*4');

    poub = fread(FID, 1, 'integer*4');
    instant(it) = fread(FID, 1, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Velo_X = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Velo_Y = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Velo_Z = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Rho = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Vort_X = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Vort_Y = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Vort_Z = fread(FID, nx*ny*nz, 'real*8');

    status = fclose(FID);

    Velo_X = reshape(Velo_X, [nx,ny,nz]);
    Velo_Y = reshape(Velo_Y, [nx,ny,nz]);
    Velo_Z = reshape(Velo_Z, [nx,ny,nz]);
    Rho    = reshape(Rho,    [nx,ny,nz]);
    Vort_X = reshape(Vort_X, [nx,ny,nz]);
    Vort_Y = reshape(Vort_Y, [nx,ny,nz]);
    Vort_Z = reshape(Vort_Z, [nx,ny,nz]);

    % we determine the kinetic energy contained in each kz plane in spectral space
    svelox = fftn(Velo_X);
    sveloy = fftn(Velo_Y);
    sveloz = fftn(Velo_Z);

    senergy = 0.5*(real(svelox).^2 + imag(svelox).^2 + real(sveloy).^2 + imag(sveloy).^2 + real(sveloz).^2 + imag(sveloz).^2);
    senergy_kz(:,it) = squeeze(sum(sum(senergy,1),2));
    [energymax(it),ind_kzmax] = max(senergy_kz(2:nz/2,it));

    kzmax(it) = kz(ind_kzmax);
end

    figure(10)
    plot(kz(2:nz/2),senergy_kz(2:nz/2,1:5:end));

    figure(20)
    plot(instant,energymax);
    title('energy max')

    figure(30)
    plot(instant,kzmax)
    title('kz max')
