% ######################
% Parameters
% ######################

% mode=1: all the instants on the same plot
% mode=2: one instant on a single plot
mode = 1

% maximum number of plots per figure (if mode=2)
subplot_par_figure = [3,2];


% ######################
% Do not change below
% ######################

if strcmp('PCWIN', computer)
	ls('.');
else
	system(['ls -t ', '.']);
end

fichier = input('\nStatistics file to use: ', 's');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reading of the statistics in the data file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% we determine the type of binary data (little-endian, big-endian) with the file header

% we start with little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(fichier, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% if the flag is not 1, we try big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(fichier, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % we check again the value of the flag
    if flag ~= 1
        fprintf(1, '\n There is a problem with the format of the data file: the flag is not 1\n');
        return
    end
end


% we read the metadata
FID = fopen(fichier, 'r',type_endian);

poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb        = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

% we display some informations about the simulation
fprintf(1, '\n Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, '\n Physical dimensions of the box: %f * %f * %f \n', lx, ly, lz);

if (stratification == 0)
    freq_brunt = 0;
else
    freq_brunt = xns;
end

fprintf(1, '\n Stratification: %f \n', freq_brunt);

if (perturb==0)
    texte = ['non perturbative'];
else
    if (lin==0) then
        texte = ['perturbative/non linear'];
    else
        texte = ['perturbative/linear'];
    end
end

% we determine the number of data for each type of shell
nb_shell_plane = nz;
nb_shell_cylinder = max(floor([nx/2+1,ny]));
nb_shell_sphere = max(floor([nx/2+1,ny,nz]));

% we read all the statistics in the data file
nb_it = 0;

poub = fread(FID, 1, 'integer*4');
clear energy_total energy_plane energy_cylinder energy_sphere;
clear enstrophy_total enstrophy_plane enstrophy_cylinder enstrophy_sphere;
clear phi1_total phi1_plane phi1_cylinder phi1_sphere;
clear phi2_total phi2_plane phi2_cylinder phi2_sphere;
clear density_fluct_total density_fluct_plane density_fluct_cylinder density_fluct_sphere;
clear instants;

while feof(FID) ~= 1
    nb_it = nb_it + 1;

    instants(nb_it) = fread(FID, 1, 'real*8');
    poub = fread(FID, 1, 'integer*4');

    poub = fread(FID, 1, 'integer*4');
    energy_total(nb_it) = fread(FID, 1, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    energy_plane(nb_it,:) = fread(FID, nb_shell_plane, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    energy_cylinder(nb_it,:) = fread(FID, nb_shell_cylinder, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    energy_sphere(nb_it,:) = fread(FID, nb_shell_sphere, 'real*8')';
    poub = fread(FID, 1, 'integer*4');

    poub = fread(FID, 1, 'integer*4');
    enstrophy_total(nb_it) = fread(FID, 1, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    enstrophy_plane(nb_it,:) = fread(FID, nb_shell_plane, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    enstrophy_cylinder(nb_it,:) = fread(FID, nb_shell_cylinder, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    enstrophy_sphere(nb_it,:) = fread(FID, nb_shell_sphere, 'real*8')';
    poub = fread(FID, 1, 'integer*4');

    poub = fread(FID, 1, 'integer*4');
    phi1_total(nb_it) = fread(FID, 1, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    phi1_plane(nb_it,:) = fread(FID, nb_shell_plane, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    phi1_cylinder(nb_it,:) = fread(FID, nb_shell_cylinder, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    phi1_sphere(nb_it,:) = fread(FID, nb_shell_sphere, 'real*8')';
    poub = fread(FID, 1, 'integer*4');

    poub = fread(FID, 1, 'integer*4');
    phi2_total(nb_it) = fread(FID, 1, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    phi2_plane(nb_it,:) = fread(FID, nb_shell_plane, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    phi2_cylinder(nb_it,:) = fread(FID, nb_shell_cylinder, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    phi2_sphere(nb_it,:) = fread(FID, nb_shell_sphere, 'real*8')';
    poub = fread(FID, 1, 'integer*4');

    poub = fread(FID, 1, 'integer*4');
    density_fluct_total(nb_it) = fread(FID, 1, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    density_fluct_plane(nb_it,:) = fread(FID, nb_shell_plane, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    density_fluct_cylinder(nb_it,:) = fread(FID, nb_shell_cylinder, 'real*8')';
    poub = fread(FID, 2, 'integer*4');
    density_fluct_sphere(nb_it,:) = fread(FID, nb_shell_sphere, 'real*8')';
    poub = fread(FID, 1, 'integer*4');

    % debut d'un nouvel enregistrement (ou eof)
    poub = fread(FID, 1, 'integer*4');
end
status = fclose(FID);

% we determine the normalized height, cylindrical and spherical radius
radius_plane = [1:nb_shell_plane]/nb_shell_plane;
radius_cylinder = [1:nb_shell_cylinder]/nb_shell_cylinder;
radius_sphere = [1:nb_shell_sphere]/nb_shell_sphere;

if mode == 1

    figure(1)
    loglog(radius_plane(2:end), energy_plane([1,4,8,12,16,20,24,28,32,36],2:end),'-s')
    xlabel('radius');
    ylabel('spherical energy');
    legend([num2str(instants([1,4,8,12,16,20,24,28,32,36])')]);
    axis square;

    title({['Plane energy density'] ; ['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  dt=',num2str(dt), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte]},'Interpreter','none');

    figure(2)

    plot(radius_plane(2:end), energy_plane([1,4,8,12,16,20,24,28,32,36],2:end))
    xlabel('radius');
    ylabel('spherical energy');
    legend([num2str(instants([1,4,8,12,16,20,24,28,32,36]))]);
    axis square;

    title({['Plane energy density'] ; ['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  dt=',num2str(dt), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte]},'Interpreter','none');


elseif mode == 2
    % we plot the different statistics
    nb_graphiques = nb_it;

    Nb_plot_par_figure = subplot_par_figure(1)*subplot_par_figure(2);
    nb_figure = ceil(nb_graphiques/Nb_plot_par_figure)

    for ind_figure = 1:nb_figure
        figure

        for it=(ind_figure-1)*Nb_plot_par_figure+1:min(ind_figure*Nb_plot_par_figure, nb_graphiques)

            subplot(subplot_par_figure(1), subplot_par_figure(2), it - (ind_figure-1)*Nb_plot_par_figure );

            loglog(radius_plane(2:end), energy_plane(:,2:end),'-s')
            xlabel('radius');
            ylabel('plane energy');
            axis square;

            title({['Plane energy density']; ['t=', num2str(instants(it)) ] });
        end
    end

end
