% Ce stript permet d'afficher les taux de croissance donnes par le code ns3d dans le fichier
% energie_quadratique_mode


% ######################
% Parametres a renseigner
% ######################

% dossier de depart dans lequel on va chercher des resultats de simulation
dir_base ='.';


% choix de ce que l'on veut tracer
% 1 : lambda(kx) pour un kz donne
% 2 : lambda(kz) pour un kx donne
% 3 : iso-contour de lambda dans le plan (kx,kz)
% 4 : lambda(kz) en moyennant l'energie sur tous les kx
% 5 : lambda(kz) en moyennant l'energie sur tous les kx en excluant kx=0
% 6 : lambda(kz) en moyennant l'energie sur tous les kx pairs d'une part et impairs d'autre part. On exclut kx=0.
choix = 4;


% vecteurs d'onde. Selon le "choix" que l'on a defini, toutes ces valeurs ne sont pas ncessaires.

% on indique l'indice de kx (compris entre 1 et kxmax+1)
kx_choisi=2;

% indice de kz par rapport  kz = 0 (kz_choisi compris entre - kzmax et kzmax)
kz_choisi=1;


% ######################
% Coeur du code
% ######################

fprintf(1, '\nRepertoire de depart : %s\n\n', dir_base);

if strcmp('PCWIN', computer)
    system(['dir /O-D /AD /B ', dir_base]);
else
    system(['ls -t ', dir_base]);
end

dir_spec = input('\nRepertoire specifique: ', 's');

fichier = [dir_base,'/',dir_spec,'/energie_quadratique_mode']


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% LECTURE DES ENERGIES DANS LE FICHIER energie_quadratique_mode
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% on determine le type de codage (little-endian, big-endian) en regardant l'en-tete du fichier

% on commence par essayer en little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(fichier, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% si le flag est different de 1, on essaye le big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(fichier, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % on verifie que le flag vaut bien 1 cette fois-ci
    if flag ~= 1
        fprintf(1, '\n Il y a un probleme dans le format du fichier : le flag est different de 1\n');
        return
    end
end


% on lit toutes les metadonnees
FID = fopen(fichier, 'r',type_endian);

poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturbe        = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

% on affiche quelques informations sur la simulation
fprintf(1, '\n Dimensions du probleme: %i * %i * %i \n', nx, ny, nz);
fprintf(1, '\n Dimensions physiques de la boite: %f * %f * %f \n', lx, ly, lz);

if (stratification == 0)
    freq_brunt = 0;
else
    freq_brunt = xns;
end

fprintf(1, '\n Stratification: %f \n', freq_brunt);

if (perturbe==0)
    texte = ['non perturbatif'];
else
    if (lin==0) then
        texte = ['perturbatif/non lineaire'];
    else
        texte = ['perturbatif/lineaire'];
    end
end


% on determine les nombres d'ondes suivant kx et kz.
% en kx, on va de 0  kxmax (kxmax+1 modes en tout).
kxmax = floor(nx/2);
kx = linspace(0, kxmax, kxmax+1)*2*pi/lx;

nzmax = floor(nz/2);
kz = zeros(1,nz);
if nz==1 & lz==0
    kz = 0;
elseif nz==1
    kz=2*pi/lz
else
    kz(1:nzmax+1) = linspace(0, nzmax, nzmax+1)*2*pi/lz;
    kz(nzmax+2:nz) = (linspace(nzmax+1, nz-1, nz-nzmax-1)-nz)*2*pi/lz;

    % on remet "dans l'ordre" les nombres d'ondes suivant z, c'est  dire qu'on va de -nzmax  +nzmax
    kz = [kz(nzmax+2:nz),kz(1:nzmax+1)];
end

% on lit toutes les energies moyennes quadratiques dans le fichier
nb_it = 0;

poub = fread(FID, 1, 'integer*4');
clear energie_mode_temp;
clear instants;
while feof(FID) ~= 1
    nb_it = nb_it + 1;

    instants(nb_it) = fread(FID, 1, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    energie_mode_temp(nb_it,:) = fread(FID, (kxmax+1)*nz, 'real*8')';
    poub = fread(FID, 1, 'integer*4');

    % debut d'un nouvel enregistrement (ou eof)
    poub = fread(FID, 1, 'integer*4');

end
status = fclose(FID);


energie_mode = zeros(nb_it,kxmax+1,nz);
for it=1:nb_it
    energie_mode(it,:,:) = reshape(energie_mode_temp(it,:),[kxmax+1,nz]);
end

% on rordonnne les taux de croissance selon les kz pour que les kz soient rangs dans l'ordre
temp = energie_mode;
energie_mode(:,:,1:nz-nzmax-1)= temp(:,:,nzmax+2:nz);
energie_mode(:,:,nz-nzmax:nz)= temp(:,:,1:nzmax+1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CALCUL des taux de croissance et affiche du resultat
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if choix == 1

    % on calcule le taux de croissance entre instants successifs
    taux_croissance = zeros(nb_it,kxmax+1,nz);

    for i=2:nb_it
        warning off MATLAB:divideByZero
        taux_croissance(i,:,:) = 0.5*log( energie_mode(i,:,:)./ energie_mode(i-1,:,:) ) / (instants(i) - instants(i-1)) ;
        warning on MATLAB:divideByZero
    end

    fprintf('Indice de kz: %i  (kz= %f)\n', kz_choisi, kz(nz-nzmax+kz_choisi));
    
    % les valeurs bouquins sont celles extraites de Michalke 1964. Elles sont valables pour une tanh avec kz=0;
    valeurs_Michalke = [1,0.9,0.8,0.7,0.6,0.5,0.4,0.3,0.2,0.1,0,0.4446;0,0.02942,0.05388,0.07305,0.08650,0.09376,0.09410,0.08654,0.06975,0.04184,0,0.09485];

    % on multiplie les valeurs par 2 parce qu'en fait dans le bouquin l'coulement de base est 0.5*(1+tanh(y)) et pas tanh(y)
    valeurs_Michalke(2,:) = 2*valeurs_Michalke(2,:);

    figure(7)    
    plot(kx, energie_mode(nb_it,:,nz-nzmax+kz_choisi), '-r*');
    xlabel('kx');
    ylabel('energie')
    title(['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte],'Interpreter','none');
   
    figure(8)   
    plot(kx, taux_croissance(nb_it,:,nz-nzmax+kz_choisi), '-r*', kx, taux_croissance(nb_it-1,:,nz-nzmax+kz_choisi), '-b*', valeurs_Michalke(1,:), valeurs_Michalke(2,:), '+r');
    xlabel('kx');
    ylabel('\lambda')
    title(['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte],'Interpreter','none');

elseif choix == 2

    % on calcule le taux de croissance entre instants successifs (nb_it-1 taux de croissance calcules)
    taux_croissance = zeros(nb_it,kxmax+1,nz);

    for i=2:nb_it
        warning off MATLAB:divideByZero
        taux_croissance(i,:,:) = 0.5*log( energie_mode(i,:,:)./ energie_mode(i-1,:,:) ) / (instants(i) - instants(i-1)) ;
        warning on MATLAB:divideByZero
    end

    fprintf('Indice de kx: %i  (kx= %f)\n', kx_choisi, kx(kx_choisi));
    
    figure(8)
    plot(kz, squeeze(energie_mode(nb_it,kx_choisi,:)), '-r*');
    xlabel('kz');
    ylabel('energie')
    title(['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte],'Interpreter','none');
    
    figure(7)
    plot(kz, squeeze(taux_croissance(nb_it,kx_choisi,:)), '-r*', kz, squeeze(taux_croissance(nb_it-1,kx_choisi,:)), '-b*');
    xlabel('kz');
    ylabel('\lambda')
    title(['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte],'Interpreter','none');

% on trace les isocontours du taux de croissance dans le plan (kx,kz)
elseif choix == 3

    % on calcule le taux de croissance entre instants successifs (nb_it-1 taux de croissance calcules)
    taux_croissance = zeros(nb_it,kxmax+1,nz);

    for i=2:nb_it
        warning off MATLAB:divideByZero
        taux_croissance(i,:,:) = 0.5*log( energie_mode(i,:,:)./ energie_mode(i-1,:,:) ) / (instants(i) - instants(i-1)) ;
        warning on MATLAB:divideByZero
    end

    figure(1)    
    contour(squeeze(kz),squeeze(kx),squeeze(taux_croissance(nb_it,:,:)),20)
    title(['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte],'Interpreter','none');
    xlabel('kz');
    ylabel('kx');
    colorbar;


    figure(2)   
    contour(squeeze(kz),squeeze(kx),squeeze(energie_mode(nb_it,:,:)),20)    
    title(['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte],'Interpreter','none');
    xlabel('kz');
    ylabel('kx');
    colorbar;
    
% on fait la moyenne suivant kx et on determine lambda(kz)
elseif choix == 4
    energie_mode_moyenne = 1/(kxmax+1)*sum(energie_mode(:,:,:),2);

    % on calcule le taux de croissance entre instants successifs
    taux_croissance = zeros(nb_it,nz);

    for i=2:nb_it
        selection = energie_mode_moyenne(i-1,:) ~= 0;
        taux_croissance(i,selection) = 0.5*log( energie_mode_moyenne(i,selection)./ energie_mode_moyenne(i-1,selection) ) / (instants(i) - instants(i-1)) ;
    end

    figure(17)
    plot(kz, taux_croissance(nb_it-1,:), '-r*', kz, taux_croissance(nb_it,:), '-b*');
    xlabel('kz')
    ylabel('\lambda')
    
    
    title({['Taux de croissance  t=', num2str(instants(nb_it))]; ['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  dt=',num2str(dt), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte]},'Interpreter','none');
    
    
    figure(16)
    hold on
    plot(kz, taux_croissance(nb_it,:), '*r');
    xlabel('kz')
    ylabel('\lambda')
    
    
    title({['Taux de croissance  t=', num2str(instants(nb_it))]; ['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  dt=',num2str(dt), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte]},'Interpreter','none');
    
    
    figure(8)
    plot(kz, energie_mode_moyenne(nb_it,:), '-b');
    xlabel('kz')
    ylabel('energie quadratique')
    title({['Energie quadratique  t=', num2str(instants(nb_it))]; ['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  dt=',num2str(dt), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte]},'Interpreter','none');

    figure(9)
    %plot(instants, taux_croissance(:,1));
    %plot(instants, [taux_croissance(:,nz-nzmax), taux_croissance(:,nz-nzmax+1)]);
    plot(instants, taux_croissance(:,nz-nzmax));
    xlabel('instants')
    ylabel('taux de croissance')
    title({'Taux de croissance au cours du temps' ; ['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  dt=',num2str(dt), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte]},'Interpreter','none');
    
    kz(nz-nzmax+1)


% on fait la moyenne suivant kx en excluant kx=0 et on determine lambda(kz)
elseif choix == 5
    energie_mode_moyenne = 1/kxmax*sum(energie_mode(:,2:kxmax+1,:),2);

    % on calcule le taux de croissance entre instants successifs (nb_it-1 taux de croissance calcules)
    taux_croissance = zeros(nb_it,nz);

    for i=2:nb_it
        selection = energie_mode_moyenne(i-1,:) ~= 0;
        taux_croissance(i,selection) = 0.5*log( energie_mode_moyenne(i,selection)./ energie_mode_moyenne(i-1,selection) ) / (instants(i) - instants(i-1)) ;
    end

    figure(7)
    plot(kz, taux_croissance(nb_it,:), '-b*', kz, taux_croissance(nb_it-1,:), '-r*');
    xlabel('kz')
    ylabel('\lambda')
    title(['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte],'Interpreter','none');
    
    figure(8)
    plot(kz, energie_mode_moyenne(nb_it,:), '-b*');
    xlabel('kz')
    ylabel('energie')
    title(['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte],'Interpreter','none');

% on moyenne d'un ct les kx pairs et de l'autre ct kx impairs. On exclut kx=0.
elseif choix == 6
    NBkx = linspace(0, kxmax, kxmax+1);

    selection = NBkx ~= 0 & mod(NBkx,2) == 0;
    energie_mode_pair = 1/(sum(selection))*sum(energie_mode(:,selection,:),2);

    selection = NBkx ~= 0 & mod(NBkx+1,2) == 0;
    energie_mode_impair = 1/(sum(selection))*sum(energie_mode(:,selection,:),2);

    taux_croissance_pair = zeros(nb_it,nz);
    taux_croissance_impair = zeros(nb_it,nz);
    
    for i=2:nb_it
        selection = energie_mode_pair(i-1,:) ~= 0;
            taux_croissance_pair(i,selection) = 0.5*log( energie_mode_pair(i,selection)./ energie_mode_pair(i-1,selection) )/(instants(i) - instants(i-1));
        
        selection = energie_mode_impair(i-1,:) ~= 0;
        taux_croissance_impair(i,selection) = 0.5*log( energie_mode_impair(i,selection)./ energie_mode_impair(i-1,selection) )/(instants(i) - instants(i-1));
    end
    

    figure(7)
    plot(kz, taux_croissance_pair(nb_it,:), '-b*', kz, taux_croissance_pair(nb_it-1,:), '-r*');
        xlabel('kz')
    ylabel('\lambda')
    title(['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte],'Interpreter','none');
    
    figure(8)
    plot(kz, taux_croissance_impair(nb_it,:), '-b*', kz, taux_croissance_impair(nb_it-1,:), '-r*');
    xlabel('kz')
    ylabel('\lambda')
    title(['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte],'Interpreter','none');
 
    figure(9)
    plot(kz, energie_mode_pair(nb_it,:), '-b*', kz, energie_mode_pair(nb_it-1,:), '-r*');
        xlabel('kz')
    ylabel('energie')
    title(['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte],'Interpreter','none');
    
    figure(10)
    plot(kz, energie_mode_impair(nb_it,:), '-b*', kz, energie_mode_impair(nb_it-1,:), '-r*');
        xlabel('kz')
    ylabel('energie')
    title(['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte],'Interpreter','none');

end