% ce script permet de tracer des plans de coupe dans le plan xz

% ######################
% global_vars a renseigner
% ######################

% start and end (0 means the last file) of the recording sequence
first_file=1
last_file=0

% on demande quel plan representer
ny0 = floor(input('\nPlan de coupe: '));

% facteur d'interpolation (on fait une interpolation FFT)
facteur_interp = 4;



% ######################
% on touche plus
% ######################

% noms des fichiers de donnees
if strcmp('PCWIN', computer)
	[poub,liste_fichiers] = system('dir /B');
else
	[poub,liste_fichiers] = system('ls ');
end

[start,finish] = regexpi(liste_fichiers,'dudz_dvdz_drhodz.t=\d*\.\d*');

nb_fichiers = size(start,2);

% we adjust the first and last file to process
if last_file == 0
    last_file=nb_fichiers;
end

clear nom_fichier
for i=1:nb_fichiers
    nom_fichier(i,:) = liste_fichiers(start(i):finish(i));
end

nom_fichier = sortrows(nom_fichier);
fichier = nom_fichier(1,:)


% we determine the type of binary data (little-endian, big-endian) with the file header

% we start with little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(fichier, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% if the flag is not 1, we try big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(fichier, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % we check again the value of the flag
    if flag ~= 1
        fprintf(1, '\n There is a problem with the format of the data file: the flag is not 1\n');
        return
    end
end


% on lit toutes les metadonnees
FID = fopen(fichier, 'r',type_endian);

poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
xnsqr           = fread(FID, 1, 'real*8');
poub            = fread(FID, 1, 'integer*4');

poub            = fread(FID, 1, 'integer*4');
instant         = fread(FID, 1, 'real*8');
poub            = fread(FID, 1, 'integer*4');

status = fclose(FID);

% on cre la grille des points de colocation
X = linspace(0, lx-lx/nx, nx);
Y = linspace(0, ly-ly/ny, ny);
Z = linspace(0, lz-lz/nz, nz);

% we display some informations about the simulation
fprintf(1, '\n Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, '\n Physical dimensions of the box: %f * %f * %f \n', lx, ly, lz);


fprintf('\n Saving images...\n');

for it=first_file:last_file
    fprintf(1, ' Num of frame: %i/%i \n', it,nb_fichiers);
    clf

	% on lit le fichier de donnees
	fprintf(1, '      Reading...');
	FID = fopen(nom_fichier(it,:),'r',type_endian);

	% on lit toutes les metadonnees d'un coup
	% on lit toutes les metadonnees d'un coup
	fseek(FID, 4*17, 'bof');
	instant = fread(FID, 1, 'real*8');

	% on va directement lire le champ en n'enregistrant que les valeurs appartenant au plan xz voulu
	for iz=1:nz
    	type_field=1;
		fseek(FID,16*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4 + 8*(iz-1)*nx*ny + 8*nx*(ny0-1), 'bof');
		dudz(:,iz) = fread(FID, nx, 'real*8');
	
		type_field=2;
		fseek(FID,16*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4 + 8*(iz-1)*nx*ny + 8*nx*(ny0-1), 'bof');
		dvdz(:,iz) = fread(FID, nx, 'real*8');
	end

    if (facteur_interp > 1)
    	fprintf(1, '\n      Interpolating fields...\n');
	
	    dudz = interpft(interpft(dudz,facteur_interp*nx,1),facteur_interp*nz,2);
	    dvdz = interpft(interpft(dvdz,facteur_interp*nx,1),facteur_interp*nz,2);
	
	    % on cre la grille des points de colocation
        X = linspace(0, lx-lx/(nx*facteur_interp), nx*facteur_interp);
        Y = linspace(0, ly-ly/(ny*facteur_interp), ny*facteur_interp);
	    Z = linspace(0, lz-lz/(nz*facteur_interp), nz*facteur_interp);
	end

	fprintf(1, '\n      Computing Rit...\n');
	Rit = xnsqr*(dudz.^2+dvdz.^2).^-1;
	clear dudz dvdz
	
	fprintf(1, '\n      Plotting...\n');
	figure(1)
	hold on
	box on
	colormap(hot(1024))
	pcolor(X,Z,Rit');
	axis equal tight   
	shading interp
	lighting phong	
	set(gca,'LineWidth',2)
	caxis([0 1])

	% on sauvegarde le fichier
	!mkdir -p crosscuts_matlab
	print('-dpng','-r600',['crosscuts_matlab/Ri_colorbar_hot.t=',num2str(instant),'.png']);
end
