% ce script permet de tracer des plans de coupe dans le plan xz

% ######################
% global_vars a renseigner
% ######################

% start and end (0 means the last file) of the recording sequence
first_file=1
last_file=0

% field to plot (vx=1,vy=2,vz=3,rho=4,wx=5,wy=6,wz=7)
type_field=4;

% on demande quel plan representer
ny0 = floor(input('\nPlan de coupe: '));


% ######################
% on touche plus
% ######################

% noms des fichiers de donnees
if strcmp('PCWIN', computer)
	[poub,liste_fichiers] = system('dir /B');
else
	[poub,liste_fichiers] = system('ls ');
end

[start,finish] = regexpi(liste_fichiers,'velo_rho_vort.t=\d*\.\d*');

nb_fichiers = size(start,2);

% we adjust the first and last file to process
if last_file == 0
    last_file=nb_fichiers;
end

clear nom_fichier
for i=1:nb_fichiers
    nom_fichier(i,:) = liste_fichiers(start(i):finish(i));
end

nom_fichier = sortrows(nom_fichier);
fichier = nom_fichier(1,:)


% we determine the type of binary data (little-endian, big-endian) with the file header

% we start with little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(fichier, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% if the flag is not 1, we try big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(fichier, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % we check again the value of the flag
    if flag ~= 1
        fprintf(1, '\n There is a problem with the format of the data file: the flag is not 1\n');
        return
    end
end

% we read the metadata
FID = fopen(fichier, 'r',type_endian);

poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb         = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

status = fclose(FID);

% we display some informations about the simulation
fprintf(1, '\n Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, '\n Physical dimensions of the box: %f * %f * %f \n', lx, ly, lz);

if (stratification == 0)
    freq_brunt = 0;
else
    freq_brunt = xns;
end

fprintf(1, '\n Stratification: %f \n', freq_brunt);

if (perturb==0)
    texte = ['non perturbative'];
else
    if (lin==0) then
        texte = ['perturbative/non linear\n'];
    else
        texte = ['perturbative/linear\n'];
    end
end

% on cree la grille des points de colocation
X = linspace(0, lx-lx/nx, nx);
Y = linspace(0, ly-ly/ny, ny);
Z = linspace(0, lz-lz/nz, nz);

fprintf('\n Saving images...\n');

for it=first_file:last_file
    fprintf(1, ' Num of frame: %i/%i \n', it,nb_fichiers);
    clf

	% on lit le fichier de donnees
	fprintf(1, '      Reading...');
	FID = fopen(nom_fichier(it,:),'r',type_endian);

	% on lit toutes les metadonnees d'un coup
	fseek(FID, 4*34, 'bof');
	instant = fread(FID, 1, 'real*8');

	% on va directement lire le champ en n'enregistrant que les valeurs appartenant au plan xz voulu
	field = zeros(nx,nz);
	for iz=1:nz
		fseek(FID,33*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4 + 8*(iz-1)*nx*ny + 8*nx*(ny0-1), 'bof');
		field(:,iz) = fread(FID, nx, 'real*8');
	end

	fprintf(1, '\n      Plotting...\n');

	figure(1)
	pcolor(X,Z,field');
	axis equal tight   
	shading interp
	lighting phong
	set(gca,'LineWidth',2,'XTick',[0 0.5 1 1.5 2], 'YTick',[0 0.2 0.4])
	%caxis([-0.02 0.02])
	colormap(jet)
		
	% on sauvegarde le fichier
	!mkdir -p crosscuts_matlab
	%print('-dpng','-r600',['crosscuts_matlab/rho.t=',num2str(instant),'.png']);
	saveas(gcf,['crosscuts_matlab/rho.t=',num2str(instant),'.fig'],'fig')
end

