! $Header: /opt/cvsroot/jmfft/lib/jmccm1d.f90,v 1.3 2004/04/01 15:48:31 teuler v8 $
! JMFFTLIB : A library of portable fourier transform subroutines
!            emulating Cray SciLib
! Author   : Jean-Marie Teuler, CNRS, teuler@lcp.u-psud.fr
!
! Permission is granted to copy and distribute this file or modified
! versions of this file for no fee, provided the copyright notice and
! this permission notice are preserved on all copies.

subroutine jmccm1d(m,n,fact,nfact,ifact,table,ntable,itable,work,nwork,ioff)

  implicit none

  ! Arguments
  integer, intent(in) :: m, n
  integer, intent(in) :: nfact, ifact
  integer, intent(in), dimension(0:nfact-1) :: fact
  integer, intent(in) :: ntable,itable
  real(kind=8), intent(in), dimension(0:ntable-1) :: table
  integer, intent(in) :: nwork
  real(kind=8), intent(inout), dimension(0:nwork-1) :: work
  integer, intent(inout) :: ioff

  ! Variables locales
  integer :: nterms
  integer :: np, pp, lastnp, premier
  integer :: nprod, nprod1, nprod2
  integer :: ny, p2, nx, p3, n5, p5
  integer :: i
  logical, save :: copyright = .false.

  ! NS3D: added to prevent the copyright from displaying
  copyright = .true.
    
  ! Comme tout le monde passe par la, on envoie le copyright (une fois)
  if (.not.copyright) then
    copyright = .true.
    print *,' '
    print *,'************************************************************'
    print *,'*       Portable Fourier transforms by JMFFTLIB            *'
    print *,'* Author : Jean-Marie Teuler, CNRS, teuler@lcp.u-psud.fr   *'
    print *,'************************************************************'
    print *,' '
  end if

  ! On recupere les facteurs
  nterms = fact(ifact)
  ny = fact(ifact+1)
  p2 = fact(ifact+2)
  nx = fact(ifact+3)
  p3 = fact(ifact+4)
  n5 = fact(ifact+5)
  p5 = fact(ifact+6)
  nprod = ny*nx*n5
  do i = 7,nterms-1,2
    nprod = nprod*fact(ifact+i+1)**fact(ifact+i)
  end do

  ! On fait nx*n5 T.F. de ny (qui est en puissances de 2)
  if (ny /= 1) then
    call jmccm1d2(p2,ny,m*(nprod/ny),table,ntable,itable,n,n/ny,work,nwork,ioff)
  end if

  ! On transpose (on tient compte de ioff) en permutant les deux parties
  ! On en profite pour multiplier par le bon wij
  if (ny /= 1 .and. nprod /= ny) then
    call jmcctranspcs(m,n,ny,nprod/ny,table,ntable,itable,work,nwork,ioff)
  end if

  ! On fait n5*ny T.F. de nx (en puissances de 3)
  if (nx /= 1) then
    call jmccm1d3(p3,nx,m*(nprod/nx),table,ntable,itable,n,n/nx,work,nwork,ioff)
  end if

  ! On transpose (on tient compte de ioff) en permutant les deux parties
  ! On en profite pour multiplier par le bon wij
  if (nx /= 1 .and. nprod /= nx) then
    call jmcctranspcs(m*ny,n,nx,nprod/(ny*nx), &
    & table,ntable,itable,work,nwork,ioff)
  end if

  ! On fait ny*nx T.F. de n5 (en puissances de 5)
  if (n5 /= 1) then
    call jmccm1d5(p5,n5,m*(nprod/n5),table,ntable,itable,n,n/n5,work,nwork,ioff)
  end if

  ! On transpose s'il y a lieu (si on a fait quelque chose et s'il reste des
  ! termes a traiter
  if (n5 /= 1 .and. nprod /= n5 .and. nterms > 7) then
    call jmcctranspcs(m*ny*nx,n,n5,nprod/(ny*nx*n5), &
    & table,ntable,itable,work,nwork,ioff)
  end if
  nprod1 = m*ny*nx
  nprod2 = ny*nx*n5
  lastnp = n5

  ! On passe aux nombres premiers autres que 2, 3 et 5
  do i = 7,nterms-1,2

    pp = fact(ifact+i)
    premier = fact(ifact+i+1)
    np = premier**pp

    call jmccm1dp(premier,pp,m*(nprod/np), &
    & table,ntable,itable,n,n/np,work,nwork,ioff)

    nprod1 = nprod1 * lastnp
    nprod2 = nprod2 * np
    if (np /= 1 .and. nprod /= np .and. nterms > i+1) then
      call jmcctranspcs(nprod1,n,np,nprod/nprod2, &
      & table,ntable,itable,work,nwork,ioff)
    end if
    lastnp = np

  end do

end subroutine jmccm1d
