module data_parser

    interface parse
        module procedure double_parse,integer_parse,logical_parse, character_parse
        module procedure array_double_parse, array_integer_parse, array_logical_parse, array_character_parse
    end interface

   contains

    subroutine double_parse(file_name,keyword,var,default_var)

        implicit none

        character(len=*), intent(in) :: file_name
        character(len=*), intent(in) :: keyword
        double precision, intent(out) :: var
        double precision, intent(in), optional :: default_var
        character(len=256) :: buffer
        character(len=256) :: temp_keyword
        integer :: eof
        logical :: found

        found = .false.

        open(unit=124,file=file_name,form="formatted",status="old",action="read",position="rewind",iostat=eof)

        do while(eof==0 .and. (.not. found))
            read(unit=124,fmt='(A256)',iostat=eof) buffer

            if (index (adjustl(buffer), keyword) == 1) then
                read(unit=buffer,fmt=*) temp_keyword, var
                found = .true.
            end if
        end do

        if ((.not. found) .and. present(default_var)) then
            var = default_var
        end if

        close(unit=124)

    end subroutine

    subroutine array_double_parse(file_name,keyword,var,default_var)

        implicit none

        character(len=*), intent(in) :: file_name
        character(len=*), intent(in) :: keyword
        double precision, dimension(:), intent(out) :: var
        double precision, dimension(:), optional :: default_var
        character(len=256) :: buffer
        character(len=256) :: temp_keyword
        integer :: eof
        logical :: found
        integer :: nb, i

        found = .false.
        nb = size(var)
        i = 0

        open(unit=124,file=file_name,form="formatted",status="old",action="read",position="rewind",iostat=eof)

        do while(eof==0 .and. (.not. found))
            read(unit=124,fmt='(A256)',iostat=eof) buffer

            if (index (adjustl(buffer), keyword) == 1) then
                i = i + 1
                read(unit=buffer,fmt=*) temp_keyword, var(i)

                if (i==nb) then
                    found = .true.
                end if
            end if
        end do

        if ((.not. found) .and. present(default_var)) then
            var(i+1:nb) = default_var(i+1:nb)
        end if

        close(unit=124)

    end subroutine


    subroutine integer_parse(file_name,keyword,var,default_var)

        implicit none

        character(len=*), intent(in) :: file_name
        character(len=*), intent(in) :: keyword
        integer, intent(out) :: var
        integer, intent(in), optional :: default_var
        character(len=256) :: buffer
        character(len=256) :: temp_keyword
        integer :: eof
        logical :: found

        found = .false.

        open(unit=124,file=file_name,form="formatted",status="old",action="read",position="rewind",iostat=eof)

        do while(eof==0 .and. (.not. found))
            read(unit=124,fmt='(A256)',iostat=eof) buffer

            if (index (adjustl(buffer), keyword) == 1) then
                read(unit=buffer,fmt=*) temp_keyword, var
                found = .true.
            end if
        end do

        if ((.not. found) .and. present(default_var)) then
            var = default_var
        end if

        close(unit=124)

    end subroutine

    subroutine array_integer_parse(file_name,keyword,var,default_var)

        implicit none

        character(len=*), intent(in) :: file_name
        character(len=*), intent(in) :: keyword
        integer, dimension(:), intent(out) :: var
        integer, dimension(:), optional :: default_var
        character(len=256) :: buffer
        character(len=256) :: temp_keyword
        integer :: eof
        logical :: found
        integer :: nb, i

        found = .false.
        nb = size(var)
        i = 0

        open(unit=124,file=file_name,form="formatted",status="old",action="read",position="rewind",iostat=eof)

        do while(eof==0 .and. (.not. found))
            read(unit=124,fmt='(A256)',iostat=eof) buffer

            if (index (adjustl(buffer), keyword) == 1) then
                i = i + 1
                read(unit=buffer,fmt=*) temp_keyword, var(i)

                if (i==nb) then
                    found = .true.
                end if
            end if
        end do

        if ((.not. found) .and. present(default_var)) then
            var(i+1:nb) = default_var(i+1:nb)
        end if

        close(unit=124)

    end subroutine


    subroutine logical_parse(file_name,keyword,var,default_var)

        implicit none

        character(len=*), intent(in) :: file_name
        character(len=*), intent(in) :: keyword
        logical, intent(out) :: var
        logical, intent(in), optional :: default_var
        character(len=256) :: buffer
        character(len=256) :: temp_keyword
        integer :: eof
        logical :: found

        found = .false.

        open(unit=124,file=file_name,form="formatted",status="old",action="read",position="rewind",iostat=eof)

        do while(eof==0 .and. (.not. found))
            read(unit=124,fmt='(A256)',iostat=eof) buffer

            if (index (adjustl(buffer), keyword) == 1) then
                read(unit=buffer,fmt=*) temp_keyword, var
                found = .true.
            end if
        end do

        if ((.not. found) .and. present(default_var)) then
            var = default_var
        end if

        close(unit=124)

    end subroutine

    subroutine array_logical_parse(file_name,keyword,var,default_var)

        implicit none

        character(len=*), intent(in) :: file_name
        character(len=*), intent(in) :: keyword
        logical, dimension(:), intent(out) :: var
        logical, dimension(:), optional :: default_var
        character(len=256) :: buffer
        character(len=256) :: temp_keyword
        integer :: eof
        logical :: found
        integer :: nb, i

        found = .false.
        nb = size(var)
        i = 0

        open(unit=124,file=file_name,form="formatted",status="old",action="read",position="rewind",iostat=eof)

        do while(eof==0 .and. (.not. found))
            read(unit=124,fmt='(A256)',iostat=eof) buffer

            if (index (adjustl(buffer), keyword) == 1) then
                i = i + 1
                read(unit=buffer,fmt=*) temp_keyword, var(i)

                if (i==nb) then
                    found = .true.
                end if
            end if
        end do

        if ((.not. found) .and. present(default_var)) then
            var(i+1:nb) = default_var(i+1:nb)
        end if

        close(unit=124)

    end subroutine


    subroutine character_parse(file_name,keyword,var,default_var)

        implicit none

        character(len=*), intent(in) :: file_name
        character(len=*), intent(in) :: keyword
        character(len=*), intent(out) :: var
        character(len=*), intent(in), optional :: default_var
        character(len=256) :: buffer
        character(len=256) :: temp_keyword
        integer :: eof
        logical :: found

        found = .false.

        open(unit=124,file=file_name,form="formatted",status="old",action="read",position="rewind",iostat=eof)

        do while(eof==0 .and. (.not. found))
            read(unit=124,fmt='(A256)',iostat=eof) buffer

            if (index (adjustl(buffer), keyword) == 1) then
                read(unit=buffer,fmt=*) temp_keyword, var
                var = adjustl(var)
                found = .true.
            end if
        end do

        if ((.not. found) .and. present(default_var)) then
            var = default_var
        end if

        close(unit=124)

    end subroutine

    subroutine array_character_parse(file_name,keyword,var,default_var)

        implicit none

        character(len=*), intent(in) :: file_name
        character(len=*), intent(in) :: keyword
        character(len=*), dimension(:), intent(out) :: var
        character(len=*), dimension(:), optional :: default_var
        character(len=256) :: buffer
        character(len=256) :: temp_keyword
        integer :: eof
        logical :: found
        integer :: nb, i

        found = .false.
        nb = size(var)
        i = 0

        open(unit=124,file=file_name,form="formatted",status="old",action="read",position="rewind",iostat=eof)

        do while(eof==0 .and. (.not. found))
            read(unit=124,fmt='(A256)',iostat=eof) buffer

            if (index (adjustl(buffer), keyword) == 1) then
                i = i + 1
                read(unit=buffer,fmt=*) temp_keyword, var(i)

                if (i==nb) then
                    found = .true.
                end if
            end if
        end do

        if ((.not. found) .and. present(default_var)) then
            var(i+1:nb) = default_var(i+1:nb)
        end if

        close(unit=124)

    end subroutine

end module
